<?php

namespace OrganizationsManagement\Controller;

use Exception;
use OrganizationsManagement\Traits\TResponse;
use Zeedhi\Framework\DTO;
use OrganizationsManagement\Service\CallCategory as CallCategoryService;
use OrganizationsManagement\Service;
use OrganizationsManagement\Util\InstanceProvider;

class CallCategory {

    use TResponse;

    /**
     * @var CallCategoryService
     */
    private $callCategoryService;

    /**
     * ConsultaOperadorSAAS constructor.
     * @param CallCategoryService $consultaOperadorSAASService
     */
    public function __construct(CallCategoryService $callCategoryService)
    {
        $this->callCategoryService = $callCategoryService;
    }

    /**
     * @param DTO\Request $request
     * @param DTO\Response $response
     * @throws Exception
     */
    public function findOrganization(DTO\Request $request, DTO\Response $response){
        try{
            $data = $this->callCategoryService->findOrganization();
            $this->sendResponse($data, 200);
        }catch(Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getCallCategory(DTO\Request $request, DTO\Response $response)
    {
        try {
            $data = $this->callCategoryService->getCallCategory();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getFindAssuntoCallCategory(DTO\Request $request, DTO\Response $response)
    {
        try{
            $data = $this->callCategoryService->getFindAssuntoCallCategory();
            $this->sendResponse($data, 200);
        } catch(\Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }

    }
    public function saveAssuntoCallCategory(DTO\Request $request, DTO\Response $response){
        try {
            $data = $request->getParameters();
            $assunto = $data["assunto"];
            $this->callCategoryService->saveAssuntoCallCategory($assunto);
            $this->sendResponse($data, 200);
        } catch (\Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
        
    }
}