<?php

namespace OrganizationsManagement\Controller;

use Exception;
use OrganizationsManagement\Traits\TResponse;
use Zeedhi\Framework\DTO;
use OrganizationsManagement\Service\OrganizationReport as OrganizationReportService;
use OrganizationsManagement\Util\Utilities;

class OrganizationReport {

    use TResponse;

    /**
     * @var OrganizationReportService
     */
    private $organizationReportService;

    /**
     * OrganizationReportService constructor.
     * @param OrganizationReportService $organizationReportService
     */
    public function __construct(OrganizationReportService $organizationReportService)
    {
        $this->organizationReportService = $organizationReportService;
    }

    /**
     * @param DTO\Request $request
     * @param DTO\Response $response
     * @throws Exception
     */
    public function organizationReport(DTO\Request $request, DTO\Response $response){
        try{
            $param = $request->getParameters();
            $page            = !empty($param['page']) ? $param['page']  : null;
            $pageSize        = !empty($param['limit']) ? $param['limit']  : null;
            $order         = !empty($param['order'])  ? $param['order']  : null;
            $dynamicFilter = !empty($param['dynamic_filter'])  ? $param['dynamic_filter'] : null;
            $data = $this->organizationReportService->organizationReport($dynamicFilter, $page, $pageSize, $order);
            $this->sendResponse($data, 200);
        }catch(Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getOrganizationReport(DTO\Request $request, DTO\Response $response){
        try{
            $param = $request->getParameters();
            $projectId = !empty($param['PROJECT_ID'])  ? $param['PROJECT_ID'] : null;
            $elementProductOrg = !empty($param['PRODUCT_ORG'])  ? $param['PRODUCT_ORG'] : null;
            $data = $this->organizationReportService->getOrganizationReport($projectId, $elementProductOrg);
            $this->sendResponse($data, 200);
        }catch(Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getOrganizations(DTO\Request $request, DTO\Response $response){
        try{
            $data = $this->organizationReportService->getOrganizations();
            $this->sendResponse($data, 200);
        }catch(Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getProducts(DTO\Request $request, DTO\Response $response){
        try{
            $data = $this->organizationReportService->getProducts();
            $this->sendResponse($data, 200);
        }catch(Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getOrganizationReportCountry(DTO\Request $request, DTO\Response $response){
        try{
            $data = $this->organizationReportService->getOrganizationReportCountry();
            $this->sendResponse($data, 200);
        }catch(Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getOrganizationReportState(DTO\Request $request, DTO\Response $response){
        try{ 
            $idPais = $request->getParameter('CDPAIS');
            $data = $this->organizationReportService->getOrganizationReportState($idPais);
            $this->sendResponse($data, 200);
        }catch(Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getOrganizationReportCity(DTO\Request $request, DTO\Response $response) {
        try{ 
            $siglaEstado = $request->getParameter('SGESTADO');
            $idPais = $request->getParameter('CDPAIS');
            $data = $this->organizationReportService->getOrganizationReportCity($siglaEstado, $idPais);
            $this->sendResponse($data, 200);
        }catch(Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }
}