<?php

namespace OrganizationsManagement\Service;

use Doctrine\ORM\EntityManager;
use OrganizationsManagement\Repository\CollectionErrors as CollectionErrorsRepository;
use OrganizationsManagement\Util\InstanceProvider;
use OrganizationsManagement\Util\Utilities;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\Libs\Util\Environment;

class CollectionErrors
{
    /**
     * @var CollectionErrorsRepository
     */
    private $collectionErrorsRepository;

    /**
     * @param CollectionErrorsRepository $collectionErrorsRepository
     */
    public function __construct(CollectionErrorsRepository $collectionErrorsRepository, Environment $environment)
    {
        $this->collectionErrorsRepository = $collectionErrorsRepository;
        $this->environment = $environment; 
    }

    public function find($parametros, $page, $limit, $order)
    {
        $nrorg = $this->environment->getNrorg();
        $nrseqcontrato = null;
        $nrparceiro = null;
        $cdtpmedidavariavel = null;
        $idelemento = null;
        $cdservorca = null;
        $idtipoorg = null;
        $idstatus = null;
        $idtpcoleta = null;
        $idsemerro = 'N';
        $order = empty($order) ? '' : $this->CollectionErrorsRepository->createOrderBy($order);

        if (isset($parametros["NRSEQCONTRATO"])) {
            $nrseqcontrato = '_';
            foreach ($parametros["NRSEQCONTRATO"] as $contrato) {
                $nrseqcontrato .= $contrato . '_';
            }
        } 

        if (isset($parametros["NRPARCNEGOCIO"])) {
            $nrparceiro = '_';
            foreach ($parametros["NRPARCNEGOCIO"] as $organizacao) {
                $nrparceiro .= $organizacao . '_';
            }
        }

        if (isset($parametros["CDTPMEDIDASERV"])) {
            $cdtpmedidavariavel = '_';
            foreach ($parametros["CDTPMEDIDASERV"] as $medidavariavel) {
                $cdtpmedidavariavel .= $medidavariavel . '_';
            }
        }

        if (isset($parametros["IDELEMENTO"])) {
            $idelemento = '-';
            foreach ($parametros["IDELEMENTO"] as $produto) {
                $idelemento .= $produto . '-';
            }
        }

        if (isset($parametros["CDSERVORCA"])) {
            $cdservorca = '_';
            foreach ($parametros["CDSERVORCA"] as $servico) {
                $cdservorca .= $servico . '_';
            }
        }

        if (isset($parametros["IDTIPOORG"])) {
            $idtipoorg = '_';
            foreach ($parametros["IDTIPOORG"] as $tipoorg) {
                $idtipoorg .= $tipoorg . '_';
            }
        }

        if (isset($parametros["IDSTATUS"])) {
            $idstatus = '_';
            foreach ($parametros["IDSTATUS"] as $status) {
                $idstatus .= $status . '_';
            }
        }

        if (isset($parametros["IDTPCOLETA"])) {
            $idtpcoleta = $parametros["IDTPCOLETA"];
        }

        if (isset($parametros["IDSEMERRO"])) {
            $idsemerro = $parametros["IDSEMERRO"];
        }

        $dataParams = array(
            "DTINICOMPET"           => $parametros["DTINICOMPET"],
            "DTFINCOMPET"           => $parametros["DTFINCOMPET"],
            "NRSEQCONTRATO"         => $nrseqcontrato,
            "NRPARCNEGOCIO"         => $nrparceiro,
            "CDTPMEDIDASERV"        => $cdtpmedidavariavel,
            "IDELEMENTO"            => $idelemento,
            "CDSERVORCA"            => $cdservorca,
            "IDTIPOORG"             => $idtipoorg,
            "IDSTATUS"              => $idstatus,
            "IDTPCOLETA"            => $idtpcoleta,
            "NRORG"                 => $nrorg,
            "IDSEMERRO"             => $idsemerro,
        );

        return $this->collectionErrorsRepository->getCollectionErros($dataParams, $page, $limit, $order);
    }

    public function getMedidasVariaveis($param)
    {
        $nrProduto = $param[0]['value'];

        if (substr($nrProduto, 0, 1) === 'G') {
            $arrayMedidas =  $this->collectionErrorsRepository->getMedidasVariaveis(array(
                "NRSEQCONTRATO"     => $param[4]['value'],
                "DTINICOMPET"       => $param[2]['value'],
                "DTFINCOMPET"       => $param[3]['value'],
                "IDELEMENTORAIZ"    => $nrProduto
            ));

            // Formatando retorno para que os nomes dos índices estejam em camelCase, conforme esperado pelo frontend. ex: measureName ao invés de MEASURENAME.
            $arrayformatado = array();

            foreach ($arrayMedidas['data'][0] as $key => $value) {
                if ($key == 'INITCOMPETENCE') {
                    $arrayformatado['data'][0]['initCompetence'] = $value;
                } elseif ($key == 'ENDCOMPETENCE') {
                    $arrayformatado['data'][0]['endCompetence'] = $value;
                } elseif ($key == 'MEASURENAME') {
                    $arrayformatado['data'][0]['measureName'] = $value;
                } elseif ($key == 'RNUM') {
                    $arrayformatado['data'][0][$key] = $value;
                } elseif ($key == 'TOTAL_LINES') {
                    $arrayformatado['data'][0][$key] = $value;
                } else {
                    $arrayformatado['data'][0][strtolower($key)] = $value;
                }
            }

            $arrayformatado['pagination'] = $arrayMedidas['pagination'];
            return $arrayformatado;
        } else {

            /* Recebe o valor faturado do front e retira do array de critérios do Mongo */
            $qtFatura = $param[5]['value'];
            unset($param[5]);

            $nrProdutoId    = substr($param[0]['value'], 2, strlen($param[0]['value']));
            $dataSourceName = null;
            $orderBy        = null;
            $cdServOrca     = $param[1]['value'];
            $page           = isset($param['page']) ? intval($param['page']) : '';
            $pageSize       = isset($param['limit']) ? intval($param['limit'])  : '';
    
            $arrayFormatado = $this->criarArrayformatado($param);
            $conditions = $this->filtraElementosArray($arrayFormatado);
            $conditionsFixed = $this->fixConditionsToZmartFilterCriteria($conditions);
            $repositoryOmProduto = InstanceProvider::getRepository()->getOmProduto();
            $conditions = $conditionsFixed['CONDITIONS'];
            $zmartFilterCriteria = InstanceProvider::getUtilities()->convertFilterCriteriaToZmartFilterCriteria($conditions, $dataSourceName);
            $organizacao = InstanceProvider::getRepository()->getOrganizacao()->findOneBy(array('nrparcnegocio' => $param[6]["value"]));
            if (empty($nrProdutoId)) {
                $productId = '0';
                $zmartFilterCriteria->addCondition('serviceId', '=', $cdServOrca);
            } else {
                $produto   = $repositoryOmProduto->find($nrProdutoId);
                $productId = strval($produto->getNrexternalProductId());
            }
    
            $zmartFilterCriteria->addCondition('productId', '=', $productId);
            $zmartFilterCriteria->setPage($page);
            $zmartFilterCriteria->setPageSize($pageSize);
    
            if (empty($orderBy)) {
                $zmartFilterCriteria->addOrderBy('date', FilterCriteria::ORDER_DESC);
            } else {
                foreach ($orderBy as $idx => $order) {
                    $zmartFilterCriteria->addOrderBy($idx, $order);
                }
            }
    
            $criteria = InstanceProvider::getMongoQueryBuilder()->buildQueryFromFilterCriteria($zmartFilterCriteria);
            $this->setMappedLikeAllCriteria($criteria, $conditionsFixed['MAPPED_LIKE_ALL']);
            $registers = InstanceProvider::getLicenseMongoDB()->aggregate("VariableMeasure_{$organizacao->getNrorg()}", $criteria);
    
            foreach ($registers as &$register) {
                
                $register['date'] = \DateTime::createFromFormat('Ymd', $register['date'])->format('d/m/Y');
                $register['endCompetence']  = \DateTime::createFromFormat('Ymd', $register['endCompetence'])->format('d/m/Y');
                $register['initCompetence'] = \DateTime::createFromFormat('Ymd', $register['initCompetence'])->format('d/m/Y');
                $contractId = Utilities::getValueInFilterDataSourceByName($conditions, 'contractId');
                $register['count'] = isset($register['count']) ? $register['count']  : '0';
                $register['countMin'] = $repositoryOmProduto->getMinContract($contractId, $nrProdutoId, $cdServOrca);
                $register['qtFatura'] = $qtFatura;
            }
    
            // $teste = '{"_id":{"$oid":"64c8c9adef5b592dc6474128"},"contractId":"13473","endCompetence":"31/08/2023","id":"43","initCompetence":"01/08/2023","productId":"4476","productName":"SaaS","operatorCode":"000000000997","operatorName":"Teknisa","measureCode":"127","measureName":"Megabyte (Limite)","contractRef":"Odhen Express","contractExpId":"296047","serviceName":null,"serviceId":"4981","date":"15/08/2023","lastDateSuccess":null,"log":[{"NOME_CLIENTE":"A CASA SERENA","SCHEMA_SAAS":"USR_ORG_4172","ESPACO_OCUPADO":"1891"}],"count":"1891","error":null,"lastStatus":"S","countMin":"30000"}';
            // $[json_decode($teste, true)];
    
            $retorno = (object) array(
                'data' =>  $registers,
                'pagination' => (object) array('page' => $page, 'limit' => $pageSize)
            );
            return $retorno;
        }
    }

    public function criarArrayformatado($conditions)
    {
        $retorno = [];
        foreach ($conditions as  $value) {
            if (is_array($value)) {
                array_push($retorno, $value);
            }
        }
        return $retorno;
    }

    public function filtraElementosArray($param)
    {
        $conditions = $param;
        foreach ($conditions as $key => $value) {
            if ((isset($value['columnName'])) && ($value['columnName'] == 'NRPRODUTO_ID' || $value['columnName'] == 'NRORG_ID' || $value['columnName'] == 'CDSERVORCA')) {
                unset($conditions[$key]);
            }
        }
        return $conditions;
    }

    private function fixConditionsToZmartFilterCriteria($conditions)
    {
        
        $mappedLikeAll = null;
        foreach ($conditions as $idx => &$condition) {

            if ((isset($condition['columnName'])) || (isset($condition['name']))){
                $name = isset($condition["name"]) ? $condition["name"] : $condition["columnName"];
                if ($condition['operator'] == 'LIKE_I' || $condition['operator'] == 'LIKE') {
                    $condition['operator'] = 'LIKE_ALL';
                }
                if ($name == 'date') {
                    $condition['value'] = array(
                        intval(\DateTime::createFromFormat('d/m/Y', $condition['value'][0])->format('Ymd')),
                        intval(\DateTime::createFromFormat('d/m/Y', $condition['value'][1])->format('Ymd'))
                    );
                } else if ($name == 'initCompetence' || $name == 'endCompetence') {
                    $condition['value'] = intval(\DateTime::createFromFormat('d/m/Y', substr($condition['value'], 0, 10))->format('Ymd'));
                }
                if ($condition['operator'] == 'MAPPED_LIKE_ALL') {
                    $mappedLikeAll = $condition;
                    unset($conditions[$idx]);
                }
            }
        }
        return array('CONDITIONS' => $conditions, 'MAPPED_LIKE_ALL' => $mappedLikeAll);
    }

    private function setMappedLikeAllCriteria(&$criteria, $mappedLikeAll)
    {
        if (!empty($mappedLikeAll)) {
            if (!isset($criteria[0]['$match'])) {
                $criteria['$match'] = array();
            }
            $name = isset($mappedLikeAll['name']) ? $mappedLikeAll['name'] : $mappedLikeAll['columnName'];
            $mappedLikeAllColumns = array('measureName', 'contractId', 'count', 'operatorName', 'serviceName');
            $value = str_replace('%', '', $mappedLikeAll['value'][0]['_ALL']);
            foreach ($mappedLikeAllColumns as $column) {
                $criteria[0]['$match']['$or'][] = array($column => array('$regex' => ".*{$value}.*"));
            }
        }
    }

    public function getElementosServOrca()
    {
        $nrorg = $this->environment->getNrorg();

        $parametros = array(
            "NRORG" => $nrorg
        );

        return $this->collectionErrorsRepository->getElementosServOrca($parametros);
    } 

    public function getSeqContratos()
    {
        $nrorg = $this->environment->getNrorg();

        $parametros = array(
            "NRORG" => $nrorg
        );

        return $this->collectionErrorsRepository->getSeqContratos($parametros);
    }

    public function getOrganizacaoParc()
    {
        $nrorg = $this->environment->getNrorg();

        $parametros = array(
            "NRORG" => $nrorg
        );

        return $this->collectionErrorsRepository->getOrganizacaoParc($parametros);
    }

    public function getCampoMedidaVariavel()
    {
        $nrorg = $this->environment->getNrorg();

        $parametros = array(
            "NRORG" => $nrorg
        );

        return $this->collectionErrorsRepository->getCampoMedidaVariavel($parametros);
    } 

    public function getCampoServOrca()
    {
        $nrorg = $this->environment->getNrorg();

        $parametros = array(
            "NRORG" => $nrorg
        );

        return $this->collectionErrorsRepository->getCampoServOrca($parametros);
    }
}
