create or replace FUNCTION BUSCA_QUANTIDADE_ESTOQUE
   (P_CDFILIAL     IN VARCHAR2,
    P_CDPRODUTO    IN VARCHAR2,
    P_NRPEDIDO_NRITEMPED IN VARCHAR2,
    P_TPEDIDO IN VARCHAR2) RETURN FLOAT
IS
    V_QTITEMPED FLOAT;
    V_QTITEMPEDEXP FLOAT;
    V_QTITEMORCAPED FLOAT;

    V_QTTOTALESTOQ FLOAT;
    V_QTDISPONIVEL FLOAT;
    V_QTRESPRODPED FLOAT;

    --V_IDCTRLDISTRIB VARCHAR2(1);
    V_IDUTILALMOX VARCHAR2(1);
    V_RESULT FLOAT;

    V_IDCNTRESTOQ VARCHAR2(1);
    --V_VRFATOCONV FLOAT;
BEGIN

    V_QTDISPONIVEL := 0;
    V_QTITEMPED := 0;

    -- busca parametrização se filial controla almoxarifado
    BEGIN
        SELECT IDUTILALMOX 
        INTO V_IDUTILALMOX 
        FROM PARAMFILIAL
        WHERE CDFILIAL = P_CDFILIAL;
    EXCEPTION WHEN NO_DATA_FOUND THEN
        V_IDUTILALMOX := 'N';
    END;

    -- busca parametrização para verificar se produto controla estoque
    BEGIN
        SELECT P.IDCNTRESTOQ
        INTO V_IDCNTRESTOQ
        FROM PRODFILI P, PRODUTO M
        WHERE (P.IDCNTRESTOQ = 'S') 
        AND (P.CDFILIAL  = P_CDFILIAL) 
        AND (P.CDPRODUTO = M.CDPRODESTO)
        AND (M.CDPRODUTO = P_CDPRODUTO);
    EXCEPTION WHEN NO_DATA_FOUND THEN
        V_IDCNTRESTOQ := 'S';
    END;

    -- entra se produto controlar estoque
    IF(V_IDCNTRESTOQ = 'S') THEN
        -- Traz quantidade de produtos em pedidos lançados (pedido ou orçamento)
        SELECT NVL(SUM(PED.QTITEMPED), 0), NVL(SUM(PED.QTITEMPEDEXP), 0), NVL(SUM(ORCA.QTITEMORCAPED), 0)
        INTO V_QTITEMPED, V_QTITEMPEDEXP, V_QTITEMORCAPED
        FROM
            (SELECT P_CDPRODUTO AS CDPRODUTO, P_CDFILIAL AS CDFILIAL, SUM(IT.QTITEMPED*NVL(PR.VRFATOCONV, 1)) AS QTITEMPED, SUM((IT.QTITEMPED - IT.QTEXPITEMPED)*NVL(PR.VRFATOCONV, 1)) AS QTITEMPEDEXP
            FROM PEDIDO PE, ITEMPEDIDO IT, PRODUTO PR, TIPOOPERAC TP
            WHERE PE.NRPEDIDO     = IT.NRPEDIDO
            AND IT.CDPRODUTO    = PR.CDPRODUTO
            AND IT.CDFILIAL     = PE.CDFILIAL
            AND PE.IDSITPEDIDO  IN ('1', '2', '6')
            AND IT.IDSITITEMPED IN ('1','2','3','7')
            AND PR.CDPRODESTO   = P_CDPRODUTO
            AND PE.CDFILIAL     = P_CDFILIAL
            AND PE.CDTIPOOPER   = TP.CDTIPOOPER(+)
            AND PE.IDENTRSAIDOP = TP.IDENTRSAIDOP(+)
            AND PE.IDTIPOPEDIDO = '1'
            AND NVL(TP.IDATUESTTPOP, 'S') = 'S'
            AND ((PE.NRPEDIDO||'_'||IT.NRITEMPED <> P_NRPEDIDO_NRITEMPED) OR (P_TPEDIDO = 'T'))
            ) PED,

            (SELECT P_CDPRODUTO AS CDPRODUTO, P_CDFILIAL AS CDFILIAL, SUM(IT.QTITEMPED*NVL(PR.VRFATOCONV, 1)) AS QTITEMORCAPED
            FROM PEDIDO PE, ITEMPEDIDO IT, PRODUTO PR, TIPOOPERAC TP, PARAMPEDIDO PED
            WHERE PE.NRPEDIDO     = IT.NRPEDIDO
            AND IT.CDPRODUTO    = PR.CDPRODUTO
            AND IT.CDFILIAL     = PE.CDFILIAL
            AND PED.CDFILIAL    = PE.CDFILIAL
            AND PE.IDSITPEDIDO  IN ('1', '2', '6')
            AND IT.IDSITITEMPED IN ('1','2','3','7')
            AND PR.CDPRODESTO   = P_CDPRODUTO
            AND PE.CDFILIAL     = P_CDFILIAL
            AND PE.CDTIPOOPER   = TP.CDTIPOOPER(+)
            AND PE.IDENTRSAIDOP = TP.IDENTRSAIDOP(+)
            AND PE.IDTIPOPEDIDO = '2'
            AND NVL(TP.IDATUESTTPOP, 'S') = 'S'
            AND NVL(PED.IDCONSORCSALDEST,'S') = 'S'
            AND ((PE.NRPEDIDO||'_'||IT.NRITEMPED <> P_NRPEDIDO_NRITEMPED) OR (P_TPEDIDO = 'T'))
            ) ORCA

            WHERE PED.CDFILIAL  = ORCA.CDFILIAL
            AND PED.CDPRODUTO = ORCA.CDPRODUTO;

        -- traz a quantidade total em estoque do produto
        SELECT SUM(RP.QTTOTALESTOQ), SUM(RP.QTDISPONIVEL), SUM(RP.QTRESPRODPED)
        INTO V_QTTOTALESTOQ, V_QTDISPONIVEL, V_QTRESPRODPED
        FROM LOCALESTOQUE LE, RESERVAPED RP, PRODUTO PR, LOTE LT, PRODFILI PF
        WHERE ((RP.CDFILIAL    = (P_CDFILIAL) OR (RP.CDFILIAL = (SELECT CDFILDEPOSITO FROM PARAMPEDIDO WHERE CDFILIAL = P_CDFILIAL AND IDUTDEPOSITO = 'S'))))
        AND PR.CDPRODUTO    = P_CDPRODUTO
        AND RP.CDPRODUTO    = PR.CDPRODESTO
        AND RP.CDFILIAL     = LE.CDFILIAL
        AND RP.CDALMOXARIFE = LE.CDALMOXARIFE
        AND RP.CDLOCALESTOQ = LE.CDLOCALESTOQ
        AND RP.CDFILIAL     = PF.CDFILIAL
        AND RP.CDPRODUTO    = PF.CDPRODUTO
        AND RP.CDALMOXARIFE = NVL('', RP.CDALMOXARIFE)
        AND RP.CDLOCALESTOQ = NVL('', RP.CDLOCALESTOQ)
        AND ((PF.IDUTCTRLLOTE = 'S') AND (TRUNC(LT.DTVALILOTE) >= TRUNC(SYSDATE)) OR ((PF.IDUTCTRLLOTE = 'N')))
        AND RP.NRLOTEESTQ   = LT.NRLOTEESTQ (+)
        AND RP.NRSUBLOTE    = LT.NRSUBLOTE  (+)
        AND RP.CDPRODUTO    = LT.CDPRODUTO  (+)
        AND RP.CDFILIAL     = LT.CDFILIAL   (+)
        AND ((V_IDUTILALMOX = 'N')
                OR ((LE.IDDISPCONPED = 'S') AND
                    (LE.CDLOCALESTOQ = (SELECT NVL(CDLOCALPRCLI, ' ')
                                          FROM PRODFILI PF
                                         WHERE (PF.CDFILIAL    = (P_CDFILIAL) OR (PF.CDFILIAL = (SELECT CDFILDEPOSITO FROM PARAMPEDIDO WHERE CDFILIAL = P_CDFILIAL AND IDUTDEPOSITO = 'S')))
                                           AND PF.CDPRODUTO    = P_CDPRODUTO
                                           AND PF.CDALMOXPRCLI = LE.CDALMOXARIFE))
                )
               );
    END IF;

    -- subtrai quantidade em pedidos lançados da quantidade total e retorna valor liquido
    IF V_QTTOTALESTOQ > 0 THEN
        V_RESULT := (V_QTTOTALESTOQ - V_QTITEMPEDEXP - V_QTITEMORCAPED);
    ELSE
        V_RESULT := 0;
    END IF;

   RETURN(V_RESULT);
END;

/
ALTER TABLE PARAMPEDIDO
ADD IDUTIESTQATUPEDAPP VARCHAR2(1) DEFAULT 'N';