import { Http, Metadata } from '@zeedhi/core';
import { Date, Form, Frame, LoadingService, Modal, ModalService, Select, Tabs, TextInput } from '@zeedhi/common';
import { TekGrid, TekRestDatasource, TekTreeGrid } from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';

export class ScoreReportController {
	private itensModal!: Modal;

	public async openModalItens({ row }: any) {
		this.itensModal = await ModalService.createFromJson('scoreReport/tabs/tabsContract/modal/itensModal', true);
		this.itensModal.show();
		this.faturamentoGridCurrentRow = row;
	}

	public async closeModalItens() {
		this.itensModal.hide();
		this.itensModal?.destroy();
	}

	public onMountedItens() {
		const itensModal = Metadata.getInstance<TekTreeGrid>('itensTekTreeGrid');
		itensModal.datasource.setFilter({
			NRORG: this.contractGridCurrentRow.NRORG,
			NRFATURACONT: this.faturamentoGridCurrentRow.NRFATURACONT,
			NRPARCNEGOCIO: this.contractGridCurrentRow.NRPARCNEGOCIO,
		});
	}

	public scoreReportGridIsVisible: boolean = true;

	public contractsGridIsVisible: boolean = true;

	get scoreReportDetailIsVisible() {
		return !this.scoreReportGridIsVisible;
	}

	public clickBackButtonScoreReport() {
		this.scoreReportGridIsVisible = true;
	}

	public clickBackButtonContracts() {
		this.contractsGridIsVisible = true;
	}

	public scoreReportGridCurrentRow: any = {};

	public contractGridCurrentRow: any = {};

	public scoreReportDetailHeaderTitle = '';

	public loadingScoreReportFinancialIsVisible = false;

	public loadingScoreReportCallIsVisible = false;

	public financialGridCurrentRow: any = {};

	public financialGridIsVisible: boolean = true;

	public NRPARCNEGOCIO: any = '';

	public firstFinancialGrid: boolean = true;

	public chamadosGridCurrentRow: any = {};

	public chamadosGridIsVisible: boolean = true;

	public produtosGridCurrentRow: any = {};

	public faturamentoGridCurrentRow: any = {};

	get financialDetailIsVisible() {
		return !this.financialGridIsVisible;
	}

	get contractsDetailIsVisible() {
		return !this.contractsGridIsVisible;
	}

	public clickBackButtonFinancial() {
		this.financialGridIsVisible = true;
	}

	public static getInstanceTekGrid(name: string): TekGrid {
		return Metadata.getInstance<TekGrid>(name);
	}

	public async organizationScoreRowClick({ row }: any) {
		const frame = Metadata.getInstance<Frame>('scoreReportFrame');
		const organizationForm = Metadata.getInstance<Form>('organizationForm');
		const operadorClienteForm = Metadata.getInstance<Form>('operadorClienteForm');
		const operadorForm = Metadata.getInstance<Form>('operadorForm');
		const financeiroForm = Metadata.getInstance<Form>('financeiroForm');
		const chamadosForm = Metadata.getInstance<Form>('chamadosForm');
		const ScoreReportTab = Metadata.getInstance<Tabs>('ScoreReportTab');
		this.scoreReportGridCurrentRow = row;
		this.scoreReportDetailHeaderTitle = row.NMORGANIZACAO;
		this.scoreReportGridIsVisible = false;
		frame.isVisible = true;
		organizationForm.value = row;
		operadorClienteForm.value = row;
		operadorForm.value = row;
		Http.get('/obtemPorId', { params: row.NRORG }).then((response) => {
			const otherValueForms = response.data.data[0];
			organizationForm.value.NRINSJUREMPR = otherValueForms.NRINSJUREMPR;
			organizationForm.value.NRINSCESTA = otherValueForms.NRINSCESTA;
			organizationForm.value.CDINSCMUNI = otherValueForms.CDINSCMUNI;
			organizationForm.value.DTINCLUSAO = otherValueForms.DTINCLUSAO;
			operadorClienteForm.value.NMOPERADOR = otherValueForms.NMOPERADOR;
			operadorClienteForm.value.DSEMAILOPER = otherValueForms.DSEMAILOPER;
		}).catch(() => {
		});
		this.loadingScoreReportFinancialIsVisible = true;
		Http.get('/getContratosForScore', { params: row }).then((response) => {
			financeiroForm.value = response.data[0];
			this.loadingScoreReportFinancialIsVisible = false;
		}).catch(() => {
			this.loadingScoreReportFinancialIsVisible = false;
		});

		this.loadingScoreReportCallIsVisible = true;
		Http.get('/getCallsForScore', { params: row }).then((responseCall) => {
			chamadosForm.value = responseCall.data;
			this.loadingScoreReportCallIsVisible = false;
		}).catch(() => {
			this.loadingScoreReportCallIsVisible = false;
		});
		// fix bug beforeChangeTab
		const params = {
			tabIndex: <Number>ScoreReportTab.activeTab,
		};
		this.beforeChangeScoreReportTab(params);
	}

	public saveOrganizationFrame() {
		const form = Metadata.getInstance<Form>('organizationComponents');
		const params = {
			NRORG: Metadata.getInstance<Select>('NRORG').value,
			NRSCORE: Metadata.getInstance<Select>('NRSCORE').value,
			IDALERTA: Metadata.getInstance<Select>('IDALERTA').value,
			IDPERCPIMP: Metadata.getInstance<Select>('IDPERCPIMP').value,
			DSDIRETORCLIE: Metadata.getInstance<Select>('DSDIRETORCLIE').value,
			DSRESPTICLIE: Metadata.getInstance<Select>('DSRESPTICLIE').value,
			CDOPERPADRINHO: Metadata.getInstance<Select>('CDOPERPADRINHO').value,
			CDOPERGERENTE: Metadata.getInstance<Select>('CDOPERGERENTE').value,
			IDOPERGERPROJ: Metadata.getInstance<Select>('IDOPERGERPROJ').value,
		};

		if (params.IDOPERGERPROJ) {
			params.IDOPERGERPROJ = 'S';
		} else {
			params.IDOPERGERPROJ = 'N';
		}
		if (Utilities.validateForm(form)) {
			LoadingService.show();
			Http.post('/saveOrganizationFrame', params).then(() => {
				Utilities.showSuccessMessage();
				Metadata.getInstance<TekGrid>('score-report-grid').reload();
			}).catch((error: any) => {
				Utilities.showErrorMessage(error);
			}).finally(() => {
				LoadingService.hide();
			});
		} else {
			Utilities.showAlertCamposObrigatorios();
		}
	}

	public getColumnStyle(params: any) {
		if (params.row.IDATIVO === 'N') {
			return 'zh-inactive';
		}
		if (params.row.IDALERTA === 'G') {
			return 'row-verde';
		}
		if (params.row.IDALERTA === 'O') {
			return 'row-laranja';
		}
		if (params.row.IDALERTA === 'R') {
			return 'row-vermelho';
		}
		if (params.row.IDALTNRSCORE === 'S') {
			return 'altnrscore-important';
		}
		if (params.row.IDPERCPIMP === 'B') {
			return 'idpercimp-bold';
		}
		return '';
	}

	public getFaturamentoStyle(params: any) {
		if (params.row.IDSITUACAO === 'FATURADA') {
			return '';
		}
		if (params.row.IDSITUACAO === 'PENDENTE') {
			return 'row-amarelo';
		}
		if (params.row.IDSITUACAO === 'ATRASO') {
			return 'row-vermelho';
		}
		if (params.row.IDSITUACAO === 'ESTIMADO') {
			return 'row-verde';
		}
		if (params.row.IDSITUACAO === 'RECEBIDO') {
			return '';
		}
		return '';
	}

	public getProductStyle(params: any) {
		let cssclass = '';
		if (params.row.ATIVO === 'P') {
			cssclass = 'om-next-grid-bold';
		} else if (params.row.ATIVO === 'N') {
			cssclass = 'om-next-product-inactive';
		} else {
			cssclass = 'om-next-product-bold';
		}
		return cssclass;
	}

	public getStyleFilter(params: any) {
		let cssclass = '';
		if (params.row.VALIDATOR === 'V') {
			cssclass = 'row-vermelho';
		} else if (params.row.VALIDATOR === 'P') {
			cssclass = 'row-verde';
		}

		if (params.row.STATUS === 'BAIXADO') {
			cssclass += ' om-next-inactive';
		}
		return cssclass;
	}

	public beforeChangeScoreReportTab({ tabIndex }: any) {
		setTimeout(() => {
			if (tabIndex === 2) {
				const financialGrid = Metadata.getInstance<TekGrid>('financialGrid');
				financialGrid.datasource.filter.NRPARCNEGOCIO = this.scoreReportGridCurrentRow.NRPARCNEGOCIO;
				if (financialGrid.datasource instanceof TekRestDatasource) {
					financialGrid.datasource.dynamicFilter = {
						IDSTATBREC: [
							{
								operation: 'IN',
								relation: 'AND',
								value: ['E', 'A', 'P'],
							},
						],
					};
				}
				financialGrid.datasource.get();
			} else if (tabIndex === 5) {
				this.setChamados();
			} else if (tabIndex === 4) {
				this.setProdutos();
			} else if (tabIndex === 3) {
				this.setContratos();
			} else if (tabIndex === 1) {
				this.setRelacionadas();
			}
		}, 1000);
	}

	public beforeChangeContractTab({ tabIndex }: any) {
		setTimeout(() => {
			if (tabIndex === 1) {
				const itemsGridContract = Metadata.getInstance<TekTreeGrid>('itemsGridContract');
				itemsGridContract.datasource.filter.NRORG = this.contractGridCurrentRow.NRORG;
				itemsGridContract.datasource.filter.NRSEQCONTRATO = this.contractGridCurrentRow.NRSEQCONTRATO;
				itemsGridContract.datasource.filter.NRSEQORCAMENTO = this.contractGridCurrentRow.NRSEQORCAMENTO;
				itemsGridContract.datasource.get();
			} else if (tabIndex === 2) {
				this.setElementsContract();
			} else if (tabIndex === 3) {
				//
			} else if (tabIndex === 4) {
				this.setFaturamento();
			}
		}, 1000);
	}

	public async financeiroRowClick({ row }: any) {
		const frame = Metadata.getInstance<Frame>('financialFrame');
		this.financialGridCurrentRow = row;
		this.financialGridIsVisible = false;
		const financialForm = Metadata.getInstance<Form>('financialForm');
		const NFForm = Metadata.getInstance<Form>('NFForm');
		const installmentForm = Metadata.getInstance<Form>('installmentForm');
		frame.isVisible = true;
		financialForm.value = row;
		NFForm.value = row;
		installmentForm.value = row;
	}

	public async setChamados() {
		const chamadosGrid = Metadata.getInstance<TekGrid>('chamadosGrid');
		chamadosGrid.datasource.filter.ORG_ID = this.scoreReportGridCurrentRow.NRORG;
		if (chamadosGrid.datasource instanceof TekRestDatasource) {
			chamadosGrid.datasource.dynamicFilter = {
				STATUS: [
					{
						operation: 'IN',
						relation: 'AND',
						value: ['O', 'P', 'W'],
					},
				],
			};
		}
		chamadosGrid.datasource.get();
	}

	public async setProdutos() {
		const produtosGrid = Metadata.getInstance<TekGrid>('produtosGrid');
		produtosGrid.datasource.filter.NRPARCNEGOCIO = this.scoreReportGridCurrentRow.NRPARCNEGOCIO;
		if (produtosGrid.datasource instanceof TekRestDatasource) {
			produtosGrid.datasource.dynamicFilter = {
				ATIVO: [
					{
						operation: 'IN',
						relation: 'AND',
						value: ['P', 'S', 'N'],
					},
				],
			};
		}
		produtosGrid.datasource.get();
	}

	public async setContratos() {
		const contratosGrid = Metadata.getInstance<TekGrid>('contratosGrid');
		contratosGrid.datasource.filter.NRPARCNEGOCIO = this.scoreReportGridCurrentRow.NRPARCNEGOCIO;
		contratosGrid.datasource.get();
	}

	public async setElementsContract() {
		const elementsContractGrid = Metadata.getInstance<TekTreeGrid>('elementsContractGrid');
		elementsContractGrid.datasource.filter.NRORG = this.contractGridCurrentRow.NRORG;
		elementsContractGrid.datasource.filter.NRSEQORCAMENTO = this.contractGridCurrentRow.NRSEQORCAMENTO;
		elementsContractGrid.datasource.get();
	}

	public async contractsRowClick({ row }: any) {
		this.contractGridCurrentRow = row;
		this.contractsGridIsVisible = false;
		this.setDadosContrato(row);
		const contractTab = Metadata.getInstance<Tabs>('contractsTab');
		const params = {
			tabIndex: <Number>contractTab.activeTab,
		};
		this.beforeChangeContractTab(params);
	}

	public setDadosContrato(row: any) {
		Metadata.getInstance<TextInput>('NRCONTRATOFCT-FormDadosContract').value = row.NRCONTRATOFCT;
		Metadata.getInstance<TextInput>('VRTOTALCONT-FormDadosContract').value = row.VRTOTALCONT;
		Metadata.getInstance<Select>('IDATIVO-FormDadosContract').value = row.IDATIVO;
		Metadata.getInstance<TextInput>('DSREFERENCIACONT-FormDadosContract').value = row.DSREFERENCIACONT;
		Metadata.getInstance<TextInput>('NMTIPOORCAMENTO-FormDadosContract').value = row.NMTIPOORCAMENTO;
		Metadata.getInstance<Date>('DTINICIOCONTRATO-FormDadosContract').value = row.DTINICIOCONTRATO.split(' ')[0];
		Metadata.getInstance<Date>('DTFIMCONTRATO-FormDadosContract').value = row.DTFIMCONTRATO.split(' ')[0];
		Metadata.getInstance<Date>('DTINIVIGENCONT-FormDadosContract').value = row.DTINIVIGENCONT.split(' ')[0];
		Metadata.getInstance<Date>('DTFINVIGENCONT-FormDadosContract').value = row.DTFINVIGENCONT.split(' ')[0];
	}

	public dataFinalFaturamento() {
		const currentDate = new globalThis.Date();
		const currentYear = currentDate.getFullYear();
		const lastDateOfYear = new globalThis.Date(currentYear, 11, 31);
		const lastDateOfYearString = [
			String(lastDateOfYear.getDate()).padStart(2, '0'),
			String(lastDateOfYear.getMonth() + 1).padStart(2, '0'),
			lastDateOfYear.getFullYear(),
		].join('/');
		return lastDateOfYearString;
	}

	public async setFaturamento() {
		const faturamentoGrid = Metadata.getInstance<TekGrid>('faturamentoGrid');
		if (faturamentoGrid.datasource instanceof TekRestDatasource) {
			faturamentoGrid.datasource.dynamicFilter = {
				IDSITUACAO: [
					{
						relation: 'AND',
						operation: 'IN',
						value: ['PENDENTE', 'ESTIMADO', 'FATURADA'],
					},
				],
				DTFINCOMPET: [
					{
						relation: 'AND',
						operation: 'EQUALS',
						value: this.dataFinalFaturamento(),
					},
				],
				NRCONTRATOFCT: [
					{
						relation: 'AND',
						operation: 'IN',
						value: this.contractGridCurrentRow.NRCONTRATOFCT,
					},
				],
			};
		}
		faturamentoGrid.datasource.setFilter({
			NRPARCNEGOCIO: this.contractGridCurrentRow.NRPARCNEGOCIO,
			NRORG: this.contractGridCurrentRow.NRORG,
		});
	}

	public async setRelacionadas() {
		const relacionadasGrid = Metadata.getInstance<TekGrid>('relacionadasGrid');
		relacionadasGrid.datasource.filter.NRORG = this.scoreReportGridCurrentRow.NRORG;
		relacionadasGrid.datasource.get();
	}
}
